Get2BoundedBuffer subclass: #AbstractStatesGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-AbsStates-Ex'!
AbstractStatesGet2BoundedBuffer comment:
'This class implements an example of inheritance anomaly.'!


!AbstractStatesGet2BoundedBuffer methodsFor: 'state predicates'!

isOne
	^contents size = 1! !

!AbstractStatesGet2BoundedBuffer methodsFor: 'default classes'!

activityClass
	"Synchronization is specified by the activity abstract states specification."

	^AbstractStatesGet2BoundedBufferActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbstractStatesGet2BoundedBuffer class
	instanceVariableNames: ''!


!AbstractStatesGet2BoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 3"
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 1/3"

	super exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio! !

IRBoundedBuffer subclass: #AbstractStatesBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-AbsStates-Ex'!
AbstractStatesBoundedBuffer comment:
'Class AbstractStatesBoundedBuffer implements the bounded buffer example with abstract states to specify synchronization.
Abstract states specification and transition is expressed within the associated activity class in order to enforce a clear separation between program and control.
Note that because abstract state specifications are quite modular, they behave quite well against inheritance anomaly.'!


!AbstractStatesBoundedBuffer methodsFor: 'default classes'!

activityClass
	"Synchronization is specified by the activity abstract states specification."

	^AbstractStatesBoundedBufferActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbstractStatesBoundedBuffer class
	instanceVariableNames: ''!


!AbstractStatesBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 3"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/3"

	super exampleSize: maxSize numberItems: numberItems speedRatio: ratio! !

AbstractStatesActivity subclass: #AbstractStatesBoundedBufferActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-AbsStates-Ex'!
AbstractStatesBoundedBufferActivity comment:
'This activity class specifies the synchronization of its associated behavior class.'!


!AbstractStatesBoundedBufferActivity methodsFor: 'abstract states'!

empty
	^#(put:)!

full
	^#(get)!

initialAbstractState
	^#empty!

partial
	"Note that we use union of sets (in fact concatenation of arrays) for better modularity.
	See class AbstractStatesGet2BoundedBufferActivity redefinition of #full,
	and consequently implicit redefinition of #partial."

	^(self empty) + (self full)! !

!AbstractStatesBoundedBufferActivity methodsFor: 'state transition'!

nextAbstractStateAfter: selector
	^oself isEmpty
		ifTrue: [#empty]
		ifFalse: [oself isFull
					ifTrue: [#full]
					ifFalse: [#partial]]! !

AbstractStatesBoundedBufferActivity subclass: #AbstractStatesGgetBoundedBufferActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-AbsStates-Ex'!
AbstractStatesGgetBoundedBufferActivity comment:
'This activity class specifies the synchronization of its associated behavior class.'!


!AbstractStatesGgetBoundedBufferActivity methodsFor: 'abstract states'!

partialNotAfterPut
	"Note that gget is specified first in order to provide it priority over get."

	^#(gget) + (self partial)! !

!AbstractStatesGgetBoundedBufferActivity methodsFor: 'state transition'!

nextAbstractStateAfter: selector
	"We check if the bounded buffer is in partial state (that is neither empty nor full) AND not after a put."

	^((oself isEmpty | oself isFull) not
			and: [selector ~= #put:])
		ifTrue: [#partialNotAfterPut]
		ifFalse: [super nextAbstractStateAfter: selector]! !

AbstractStatesBoundedBufferActivity subclass: #AbstractStatesGet2BoundedBufferActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-AbsStates-Ex'!
AbstractStatesGet2BoundedBufferActivity comment:
'This activity class specifies the synchronization of its associated behavior class.'!


!AbstractStatesGet2BoundedBufferActivity methodsFor: 'abstract states'!

full
	^(super full) + #(get2)!

one
	^(self partial) - #(get2)

	"Note that an alternative is: ^(self empty) + #(get)."! !

!AbstractStatesGet2BoundedBufferActivity methodsFor: 'state transition'!

nextAbstractStateAfter: selector
	^oself isOne
		ifTrue: [#one]
		ifFalse: [super nextAbstractStateAfter: selector]! !

GgetBoundedBuffer subclass: #AbstractStatesGgetBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-AbsStates-Ex'!
AbstractStatesGgetBoundedBuffer comment:
'This class implements an example of inheritance anomaly.'!


!AbstractStatesGgetBoundedBuffer methodsFor: 'default classes'!

activityClass
	"Synchronization is specified by the activity abstract states specification."

	^AbstractStatesGgetBoundedBufferActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbstractStatesGgetBoundedBuffer class
	instanceVariableNames: ''!


!AbstractStatesGgetBoundedBuffer class methodsFor: 'example'!

exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGgetSize: 3 numberItems: 8 speedRatio: 3"
	"self exampleGgetSize: 3 numberItems: 8 speedRatio: 1/3"

	"Note that there is no guarantee that all gget requests will be served.
	As a result the bounded buffer may stop before all producer and consumer requests had been served."

	super exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio! !
